<?php
require_once 'config.php';
header('Content-Type: application/json');
header('Access-Control-Allow-Origin: *');  // Permite solicitudes de cualquier origen
header('Access-Control-Allow-Methods: GET, POST, PUT');  // Permite métodos específicos
header('Access-Control-Allow-Headers: Content-Type, Authorization');  // Permite headers específicos

// Manejo inicial para las solicitudes OPTIONS
if ($_SERVER['REQUEST_METHOD'] === 'OPTIONS') {
    // Responde con éxito sin contenido adicional
    http_response_code(204); // No Content
    exit();
}

$options = [
    PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
    PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC,
    PDO::ATTR_EMULATE_PREPARES => false,
];

$dsn = "mysql:host=" . DB_HOSTNAME . ";port=" . DB_PORT . ";dbname=" . DB_DATABASE . ";charset=utf8mb4";
try {
    $pdo = new PDO($dsn, DB_USERNAME, DB_PASSWORD, $options);
} catch (PDOException $e) {
    http_response_code(500);
    echo json_encode(['error' => "Error de conexión: " . $e->getMessage()]);
    exit;
}

if (!isset($_SERVER['PHP_AUTH_USER'])) {
    header('WWW-Authenticate: Basic realm="Mi API"');
    header('HTTP/1.0 401 Unauthorized');
    echo json_encode(['error' => 'Autenticación requerida.']);
    exit;
} else {
    if ($_SERVER['PHP_AUTH_USER'] !== 'Super' || $_SERVER['PHP_AUTH_PW'] !== 'Lmas2020!1') {
        header('HTTP/1.0 401 Unauthorized');
        echo json_encode(['error' => 'Credenciales no válidas.']);
        exit;
    }
}

$method = $_SERVER['REQUEST_METHOD'];

switch ($method) {
    case 'POST':
        // Lógica para manejar POST
        break;
    case 'GET':
        // Lógica para manejar GET
        break;
    case 'PUT':
        $input = json_decode(file_get_contents("php://input"), true);
        if (isset($input['sectionid'], $input['prodid'], $input['optid'], $input['optsessionid'], $input['optionqty'])) {
            $sql = "UPDATE `oczv_optionsaved` SET `optionqty` = :optionqty WHERE `sectionid` = :sectionid AND `prodid` = :prodid AND `optid` = :optid AND `optsessionid` = :optsessionid";
            $params = [
                ':sectionid' => $input['sectionid'],
                ':prodid' => $input['prodid'],
                ':optid' => $input['optid'],
                ':optsessionid' => $input['optsessionid'],
                ':optionqty' => $input['optionqty'],
            ];

            $stmt = $pdo->prepare($sql);
            if ($stmt->execute($params)) {
                echo json_encode(['success' => 'Datos actualizados correctamente.']);
            } else {
                echo json_encode(['error' => 'Error al actualizar los datos.']);
            }
        } else {
            http_response_code(400);
            echo json_encode(['error' => 'Faltan parámetros necesarios para la actualización.']);
        }
        break;
    default:
        http_response_code(405);
        echo json_encode(['error' => 'Método HTTP no soportado']);
        break;
}
?>
