<?php
require_once 'config.php';  // Asegúrate de que la ruta al archivo de configuración es correcta.

header('Content-Type: application/json');
header('Access-Control-Allow-Origin: *');  // Permite solicitudes de cualquier origen
header('Access-Control-Allow-Methods: GET, OPTIONS');  // Ahora incluye OPTIONS
header('Access-Control-Allow-Headers: Content-Type, Authorization');  // Especifica las cabeceras permitidas

// Si la solicitud es OPTIONS, envía los headers y termina la ejecución
if ($_SERVER['REQUEST_METHOD'] == 'OPTIONS') {
    http_response_code(204);  // No Content
    exit();
}

// Configuración de conexión a la base de datos usando PDO
$options = [
    PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
    PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC,
    PDO::ATTR_EMULATE_PREPARES => false,
];

$dsn = "mysql:host=" . DB_HOSTNAME . ";port=" . DB_PORT . ";dbname=" . DB_DATABASE . ";charset=utf8mb4";
try {
    $pdo = new PDO($dsn, DB_USERNAME, DB_PASSWORD, $options);
} catch (PDOException $e) {
    http_response_code(500);
    echo json_encode(['error' => "Error de conexión: " . $e->getMessage()]);
    exit;
}

// Autenticación básica
if (!isset($_SERVER['PHP_AUTH_USER'])) {
    header('WWW-Authenticate: Basic realm="Mi API"');
    header('HTTP/1.0 401 Unauthorized');
    echo json_encode(['error' => 'Autenticación requerida.']);
    exit;
} else {
    if ($_SERVER['PHP_AUTH_USER'] !== 'Super' || $_SERVER['PHP_AUTH_PW'] !== 'Lmas2020!1') {
        header('HTTP/1.0 401 Unauthorized');
        echo json_encode(['error' => 'Credenciales no válidas.']);
        exit;
    }
}

// Recolección y validación de parámetros GET
$prodid = $_GET['prodid'] ?? null;
$optionnume = $_GET['optionnume'] ?? null;
$optid = $_GET['optid'] ?? null;

if (is_null($prodid) || is_null($optionnume) || is_null($optid)) {
    http_response_code(400);
    echo json_encode(['error' => 'Faltan parámetros necesarios para la consulta.']);
    exit;
}

// Ejecución de consulta SQL segura con parámetros
try {
    $sql = "SELECT * FROM `oczv_option_qty` WHERE `prodid` = :prodid AND `optionnume` = :optionnume AND `optid` = :optid";
    $stmt = $pdo->prepare($sql);
    $stmt->execute([
        ':prodid' => $prodid,
        ':optionnume' => $optionnume,
        ':optid' => $optid
    ]);
    $results = $stmt->fetchAll();
    if (count($results) > 0) {
        echo json_encode(['data' => $results]);
    } else {
        echo json_encode(['error' => 'No se encontraron datos.']);
    }
} catch (PDOException $e) {
    http_response_code(500);
    echo json_encode(['error' => 'Error al ejecutar la consulta: ' . $e->getMessage()]);
    exit;
}
?>
