<?php
require_once 'config.php';  // Asegúrate de que la ruta al archivo de configuración es correcta.

header('Content-Type: application/json');

// Conexión a la base de datos usando PDO
$options = [
    PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
    PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC,
    PDO::ATTR_EMULATE_PREPARES => false,
];

$dsn = "mysql:host=" . DB_HOSTNAME . ";port=" . DB_PORT . ";dbname=" . DB_DATABASE . ";charset=utf8mb4";
try {
    $pdo = new PDO($dsn, DB_USERNAME, DB_PASSWORD, $options);
} catch (PDOException $e) {
    http_response_code(500);
    echo json_encode(['error' => "Error de conexión: " . $e->getMessage()]);
    exit;
}

// Autenticación básica
if (!isset($_SERVER['PHP_AUTH_USER'])) {
    header('WWW-Authenticate: Basic realm="Mi API"');
    header('HTTP/1.0 401 Unauthorized');
    echo json_encode(['error' => 'Autenticación requerida.']);
    exit;
} else {
    // Comprueba los datos de usuario y contraseña recibidos
    if ($_SERVER['PHP_AUTH_USER'] !== 'Super' || $_SERVER['PHP_AUTH_PW'] !== 'Lmas2020!1') {
        header('HTTP/1.0 401 Unauthorized');
        echo json_encode(['error' => 'Credenciales no válidas.']);
        exit;
    }
}

$method = $_SERVER['REQUEST_METHOD'];
$input = json_decode(file_get_contents('php://input'), true);

if ($method !== 'POST') {
    http_response_code(405); // Método no permitido
    echo json_encode(['error' => 'Método HTTP no soportado']);
    exit;
}

// Validación de datos de entrada
$required_fields = ['sectionid', 'optiontype', 'optionnume', 'optionqty', 'optionselected', 'optionname', 'optionimage', 'optiondatetime', 'optionprice', 'terminal', 'optionstatus', 'prodid']; // Agrega 'prodid' a los campos requeridos
$data = [];
foreach ($required_fields as $field) {
    if (!isset($input[$field])) {
        http_response_code(400); // Solicitud incorrecta
        echo json_encode(['error' => "Falta el campo requerido: $field"]);
        exit;
    }
    $data[$field] = $input[$field];
}

// `optid` es opcional, añadir solo si está presente
if (isset($input['optid'])) {
    $data['optid'] = $input['optid'];
}

try {
    // Preparar y ejecutar la consulta
    $query = "INSERT INTO `oczv_optionqty` (`sectionid`, `optiontype`, `optionnume`, `optionqty`, `optionselected`, `optionname`, `optionimage`, `optiondatetime`, `optionprice`, `terminal`, `optionstatus`, `prodid`" . (isset($data['optid']) ? ", `optid`" : "") . ") VALUES (:sectionid, :optiontype, :optionnume, :optionqty, :optionselected, :optionname, :optionimage, :optiondatetime, :optionprice, :terminal, :optionstatus, :prodid" . (isset($data['optid']) ? ", :optid" : "") . ")";
    $stmt = $pdo->prepare($query);
    $stmt->execute($data);
    echo json_encode(['result' => 'success', 'id' => $pdo->lastInsertId()]);
} catch (PDOException $e) {
    http_response_code(500);
    echo json_encode(['error' => 'Error al ejecutar la consulta: ' . $e->getMessage()]);
    exit;
}
?>
