<?php
// Configuración de CORS
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: GET, POST, OPTIONS");
header("Access-Control-Allow-Headers: Origin, X-Requested-With, Content-Type, Accept, Authorization");

ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

require_once('config.php');

function log_message($message) {
    $log_file = __DIR__ . '/log_file.log'; // Usar una ruta relativa en el mismo directorio que el script
    error_log($message, 3, $log_file);
}

log_message("Request received\n");

// Verificación de autenticación
if (!isset($_SERVER['PHP_AUTH_USER']) || !isset($_SERVER['PHP_AUTH_PW']) || 
    $_SERVER['PHP_AUTH_USER'] !== API_USERNAME || $_SERVER['PHP_AUTH_PW'] !== API_PASSWORD) {
    header('WWW-Authenticate: Basic realm="My Realm"');
    header('HTTP/1.0 401 Unauthorized');
    echo 'Unauthorized';
    log_message("Invalid credentials provided\n");
    exit;
}

log_message("Valid credentials provided\n");

if ($_SERVER['REQUEST_METHOD'] == 'POST') {
    log_message("Processing POST request\n");

    // Datos adicionales enviados como parámetros separados
    $orderNumber = isset($_POST['orderNumber']) ? $_POST['orderNumber'] : 'N/A';
    $customerName = isset($_POST['customerName']) ? $_POST['customerName'] : 'N/A';
    $mobileNumber = isset($_POST['mobileNumber']) ? $_POST['mobileNumber'] : 'N/A';
    $paymentMethod = isset($_POST['paymentMethod']) ? $_POST['paymentMethod'] : 'N/A';
    $serviceTo = isset($_POST['serviceTo']) ? $_POST['serviceTo'] : 'N/A';
    $orderStatus = isset($_POST['orderStatus']) ? $_POST['orderStatus'] : 'N/A';
    $businessName = isset($_POST['businessName']) ? $_POST['businessName'] : 'My Nombre De Compania';
    $businessAddress = isset($_POST['businessAddress']) ? $_POST['businessAddress'] : '25 Main Street';
    $businessCityStateZip = isset($_POST['businessCityStateZip']) ? $_POST['businessCityStateZip'] : 'New York NY 12588';
    $businessPhone = isset($_POST['businessPhone']) ? $_POST['businessPhone'] : '(555) 555-5555';
    $thankYouMessage = isset($_POST['thankYouMessage']) ? $_POST['thankYouMessage'] : 'Thank You For Shopping!';

    log_message("Received additional parameters: orderNumber=$orderNumber, customerName=$customerName, mobileNumber=$mobileNumber, paymentMethod=$paymentMethod, serviceTo=$serviceTo, orderStatus=$orderStatus, businessName=$businessName, businessAddress=$businessAddress, businessCityStateZip=$businessCityStateZip, businessPhone=$businessPhone, thankYouMessage=$thankYouMessage\n");

    // Datos JSON enviados en el campo 'data'
    if (isset($_POST['data'])) {
        log_message("Received data field\n");
    } else {
        log_message("Data field not received\n");
    }

    $jsonData = isset($_POST['data']) ? $_POST['data'] : null;
    log_message("JSON Data: " . $jsonData . "\n");
    $response = json_decode($jsonData, true);

    if (json_last_error() !== JSON_ERROR_NONE) {
        log_message("JSON decode error: " . json_last_error_msg() . "\n");
        die("Error: JSON decode error - " . json_last_error_msg());
    }

    if (!$response) {
        log_message("Error decoding JSON data\n");
        die("Error: Respuesta de API no válida.");
    }

    log_message("JSON data decoded successfully\n");

    if (!isset($response['success']) || $response['success'] != 1) {
        log_message("API response indicates failure\n");
        die("Error: Respuesta de API no válida.");
    }

    $data = isset($response['data']) ? $response['data'] : null;
    $products = isset($data['products']) ? $data['products'] : [];
    $totals = isset($data['totals']) ? $data['totals'] : [];

    if (empty($products) || empty($totals)) {
        log_message("Products or totals are empty\n");
        die("Error: 'products' o 'totals' no pueden estar vacíos.");
    }

    log_message("Products and totals received successfully\n");

    // Obtener la fecha y hora actual en formato USA
    $currentDate = date('m/d/Y');
    $currentTime = date('h:i:s A');

    // Generar contenido del recibo en HTML
    $htmlContent = '<html><head><style>
    body { font-family: Arial, sans-serif; }
    .center { text-align: center; }
    .bold { font-weight: bold; }
    .header, .footer { margin: 20px 0; }
    .line { border-bottom: 1px solid #000; margin: 10px 0; }
    .product { display: flex; justify-content: space-between; }
    .total { display: flex; justify-content: space-between; font-weight: bold; }
    .columns { display: flex; justify-content: space-between; }
    .description { flex: 2; }
    .qty, .price { flex: 1; text-align: right; }
    </style></head><body>';

    // Encabezado del negocio
    $htmlContent .= '<div class="header center bold" style="font-size: 1.5em;">' . $businessName . '</div>';
    $htmlContent .= '<div class="center">' . $businessAddress . '<br>' . $businessCityStateZip . '<br>' . $businessPhone . '</div>';
    $htmlContent .= '<div class="line"></div>';

    // Información del pedido
    $htmlContent .= '<div>Order Number: ' . $orderNumber . '</div>';
    $htmlContent .= '<div>Customer Name: ' . $customerName . '</div>';
    $htmlContent .= '<div>Mobile Number: ' . $mobileNumber . '</div>';
    $htmlContent .= '<div>Payment Method: ' . $paymentMethod . '</div>';
    $htmlContent .= '<div>Service To: ' . $serviceTo . '</div>';
    $htmlContent .= '<div>Order Status: ' . $orderStatus . '</div>';
    $htmlContent .= '<div class="line"></div>';

    // Fecha y hora de impresión
    $htmlContent .= '<div>Date: ' . $currentDate . '</div>';
    $htmlContent .= '<div>Time: ' . $currentTime . '</div>';
    $htmlContent .= '<div class="line"></div>';

    // Encabezado de las columnas
    $htmlContent .= '<div class="columns bold"><span class="description">Description</span><span class="qty">Qty</span><span class="price">Price</span></div>';
    $htmlContent .= '<div class="line"></div>';

    // Productos
    foreach ($products as $product) {
        $htmlContent .= '<div class="product"><span class="description">' . htmlspecialchars($product['name']) . '</span><span class="qty">' . htmlspecialchars($product['quantity']) . '</span><span class="price">' . htmlspecialchars($product['price']) . '</span></div>';
        // Añadir opciones debajo de la descripción del producto
        foreach ($product['option'] as $option) {
            $htmlContent .= '<div class="product"><span class="description" style="margin-left: 20px;">' . htmlspecialchars($option['value']) . '</span></div>';
        }
    }

    $htmlContent .= '<div class="line"></div>';

    // Totales
    foreach ($totals as $total) {
        $htmlContent .= '<div class="total"><span class="description">' . htmlspecialchars($total['title']) . '</span><span class="price">' . '$' . number_format($total['value'], 2) . '</span></div>';
    }

    $htmlContent .= '<div class="footer center bold">' . $thankYouMessage . '</div>';
    $htmlContent .= '</body></html>';

    // Generar un nombre de archivo seguro basado en el número de orden
    $orderNumberSafe = preg_replace('/[^a-zA-Z0-9_-]/', '_', $orderNumber);

    // Guardar el contenido HTML en un archivo en el directorio 'receipts'
    $receiptsDir = __DIR__ . '/receipts';
    if (!is_dir($receiptsDir)) {
        mkdir($receiptsDir, 0755, true);
    }

    $filePath = $receiptsDir . '/receipt_' . $orderNumberSafe . '.html';
    file_put_contents($filePath, $htmlContent);

    echo "Recibo generado y guardado en $filePath";

    // Aquí comienza la sección para la impresión
    // Comandos ESC/POS
    $esc = "\x1b"; // ESC byte en notación hexadecimal
    $newLine = "\x0a"; // LF byte en notación hexadecimal
    $centerAlign = $esc . "\x61\x01"; // Centrar texto
    $leftAlign = $esc . "\x61\x00"; // Alinear texto a la izquierda
    $boldOn = $esc . "\x45\x01"; // Negrita encendida
    $boldOff = $esc . "\x45\x00"; // Negrita apagada
    $fontSizeDouble = "\x1d\x21\x11"; // Doble tamaño de fuente
    $fontSizeNormal = "\x1d\x21\x00"; // Tamaño normal de fuente
    $fontSizeReduced = "\x1d\x21\x01"; // Tamaño reducido de fuente
    $cutPaper = "\x1d\x56\x42\x00"; // Comando de corte de papel
    $dataStr = $esc . "@"; // Inicializar impresora

    // Encabezado del negocio en negrita
    $dataStr .= $centerAlign;
    $dataStr .= $boldOn;
    $dataStr .= $fontSizeDouble;
    $dataStr .= $businessName . $newLine;
    $dataStr .= $fontSizeNormal;
    $dataStr .= $businessAddress . $newLine;
    $dataStr .= $businessCityStateZip . $newLine;
    $dataStr .= $businessPhone . $newLine;
    $dataStr .= $boldOff;
    $dataStr .= $newLine . $newLine . $newLine . $newLine; // Cuatro líneas en blanco
    $dataStr .= str_repeat("-", 48) . $newLine; // Línea separadora

    // Información del pedido
    $dataStr .= $leftAlign;
    $dataStr .= "Order Number: " . $orderNumber . $newLine;
    $dataStr .= "Customer Name: " . $customerName . $newLine;
    $dataStr .= "Mobile Number: " . $mobileNumber . $newLine;
    $dataStr .= "Payment Method: " . $paymentMethod . $newLine;
    $dataStr .= "Service To: " . $serviceTo . $newLine;
    $dataStr .= "Order Status: " . $orderStatus . $newLine;
    $dataStr .= str_repeat("-", 48) . $newLine; // Línea separadora

    // Fecha y hora de impresión
    $dataStr .= "Date: " . $currentDate . $newLine;
    $dataStr .= "Time: " . $currentTime . $newLine;
    $dataStr .= str_repeat("-", 48) . $newLine; // Línea separadora

    // Encabezado de las columnas
    $dataStr .= $boldOn;
    $dataStr .= str_pad("Description", 24) . str_pad("Qty", 12, ' ', STR_PAD_LEFT) . str_pad("Price", 12, ' ', STR_PAD_LEFT) . $newLine;
    $dataStr .= $boldOff;
    $dataStr .= str_repeat("-", 48) . $newLine; // Línea separadora

    // Productos
    foreach ($products as $product) {
        $descripcion = str_pad(substr($product['name'], 0, 24), 24);
        $cantidad = str_pad($product['quantity'], 12, ' ', STR_PAD_LEFT);
        $precio = str_pad($product['price'], 12, ' ', STR_PAD_LEFT);
        $dataStr .= $boldOn . $descripcion . $boldOff . $cantidad . $precio . $newLine;

        // Añadir opciones debajo de la descripción del producto
        foreach ($product['option'] as $option) {
            $optionValue = "   " . $option['value']; // Comienza con 3 espacios
            $dataStr .= $optionValue . $newLine;
        }
    }

    $dataStr .= str_repeat("-", 48) . $newLine; // Línea separadora

    // Totales
    foreach ($totals as $total) {
        $title = str_pad($total['title'], 24);
        $value = str_pad("$" . number_format($total['value'], 2), 24, ' ', STR_PAD_LEFT);
        if (strpos($total['title'], 'Sub-Total') !== false || strpos($total['title'], 'Total') !== false) {
            $dataStr .= $boldOn . $fontSizeReduced . $title . $value . $fontSizeNormal . $boldOff . $newLine;
        } else {
            $dataStr .= $title . $value . $newLine;
        }
    }

    // Espacio adicional para QR y corte de papel
    $dataStr .= $esc . "d" . "\x02"; // Avanzar 2 líneas

    // Código QR centrado
    $dataStr .= $centerAlign; // Centrar el QR
    $qrData = "https://dgcana.com/ocart2";
    $modelQR = "\x1d\x28\x6b\x04\x00\x31\x41\x32\x00"; // Selección de modelo de QR
    $sizeQR = "\x1d\x28\x6b\x03\x00\x31\x43\x08"; // Tamaño del módulo de QR (8)
    $errorQR = "\x1d\x28\x6b\x03\x00\x31\x45\x30"; // Nivel de corrección de error (48 - L)
    $storeLen = strlen($qrData) + 3;
    $storeQR = "\x1d\x28\x6b" . chr($storeLen & 0xFF) . chr(($storeLen >> 8) & 0xFF) . "\x31\x50\x30" . $qrData; // Almacenar datos del QR
    $printQR = "\x1d\x28\x6b\x03\x00\x31\x51\x30"; // Imprimir el QR

    $dataStr .= $modelQR . $sizeQR . $errorQR . $storeQR . $printQR;

    // Texto "Thank You For Shopping!" en negrita
    $dataStr .= $newLine . $newLine . $newLine; // Tres líneas de espacio
    $dataStr .= $centerAlign . $boldOn . $thankYouMessage . $boldOff . $newLine; // Texto centrado en negrita
    $dataStr .= str_repeat(" ", 5) . $newLine; // Cinco espacios

    // Corte de papel
    $dataStr .= $cutPaper; // Comando de corte de papel

    // IP y puerto de la impresora
    $printerIp = PRINTER_IP;
    $printerPort = PRINTER_PORT;

    function sendToPrinter($printerIp, $printerPort, $dataStr) {
        echo "Creando socket...\n";
        $socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
        if ($socket === false) {
            log_message("Error al crear el socket: " . socket_strerror(socket_last_error()) . "\n");
            die("Error al crear el socket: " . socket_strerror(socket_last_error()));
        }

        echo "Conectando al socket...\n";
        $result = socket_connect($socket, $printerIp, $printerPort);
        if ($result === false) {
            log_message("Error al conectar al socket: " . socket_strerror(socket_last_error($socket)) . "\n");
            die("Error al conectar al socket: " . socket_strerror(socket_last_error($socket)));
        }

        echo "Escribiendo en el socket...\n";
        socket_write($socket, $dataStr, strlen($dataStr));
        socket_close($socket);
        echo "Impresión completada.\n";
    }

    try {
        sendToPrinter($printerIp, $printerPort, $dataStr);
        echo "Ticket enviado a la impresora.";
    } catch (Exception $e) {
        log_message("Error: " . $e->getMessage() . "\n");
        echo "Error: " . $e->getMessage();
    }
} else {
    echo "Método no soportado.";
}
?>
