<?php
require_once '../config.php';

header('Content-Type: application/json');
header('Access-Control-Allow-Origin: *');
header('Access-Control-Allow-Methods: PUT, OPTIONS'); // Añadir OPTIONS a los métodos permitidos
header('Access-Control-Allow-Headers: Content-Type, Authorization');

if ($_SERVER['REQUEST_METHOD'] === 'OPTIONS') {
    http_response_code(204); // No Content, adecuado para preflight
    exit();
}

$options = [
    PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
    PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC,
    PDO::ATTR_EMULATE_PREPARES => false,
];

$dsn = "mysql:host=" . DB_HOSTNAME . ";port=" . DB_PORT . ";dbname=" . DB_DATABASE . ";charset=utf8mb4";
try {
    $pdo = new PDO($dsn, DB_USERNAME, DB_PASSWORD, $options);
} catch (PDOException $e) {
    http_response_code(500);
    echo json_encode(['error' => "Error de conexión: " . $e->getMessage()]);
    exit;
}

if (!isset($_SERVER['PHP_AUTH_USER'])) {
    header('WWW-Authenticate: Basic realm="Mi API"');
    header('HTTP/1.0 401 Unauthorized');
    echo json_encode(['error' => 'Autenticación requerida.']);
    exit;
} else {
    if ($_SERVER['PHP_AUTH_USER'] !== 'Super' || $_SERVER['PHP_AUTH_PW'] !== 'Lmas2020!1') {
        header('HTTP/1.0 401 Unauthorized');
        echo json_encode(['error' => 'Credenciales no válidas.']);
        exit;
    }
}

$input = json_decode(file_get_contents("php://input"), true);

if (!isset($input['prodid'], $input['optionnume'], $input['optid'], $input['min'], $input['max'], $input['stock'])) {
    http_response_code(400);
    echo json_encode(['error' => 'Faltan parámetros necesarios para la actualización.']);
    exit;
}

try {
    $sql = "UPDATE `ocow_option_qty` SET `min` = :min, `max` = :max, `stock` = :stock WHERE `prodid` = :prodid AND `optionnume` = :optionnume AND `optid` = :optid";
    $stmt = $pdo->prepare($sql);
    $params = [
        ':min' => $input['min'],
        ':max' => $input['max'],
        ':stock' => $input['stock'],
        ':prodid' => $input['prodid'],
        ':optionnume' => $input['optionnume'],
        ':optid' => $input['optid'],
    ];
    $stmt->execute($params);
    if ($stmt->rowCount() > 0) {
        echo json_encode(['success' => 'Datos actualizados correctamente.']);
    } else {
        echo json_encode(['error' => 'No se encontraron registros para actualizar.']);
    }
} catch (PDOException $e) {
    http_response_code(500);
    echo json_encode(['error' => 'Error al ejecutar la consulta: ' . $e->getMessage()]);
    exit;
}
?>
