<?php
require_once '../config.php';  // Asegúrate de que la ruta al archivo de configuración es correcta.
header('Content-Type: application/json');
header('Access-Control-Allow-Origin: *');
header('Access-Control-Allow-Methods: DELETE, OPTIONS');  // Incluye OPTIONS en los métodos permitidos
header('Access-Control-Allow-Headers: Content-Type, Authorization');

if ($_SERVER['REQUEST_METHOD'] === 'OPTIONS') {
    http_response_code(204); // No Content, respuesta adecuada para preflight
    exit();
}

// Conexión a la base de datos usando PDO
$options = [
    PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
    PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC,
    PDO::ATTR_EMULATE_PREPARES => false,
];

$dsn = "mysql:host=" . DB_HOSTNAME . ";port=" . DB_PORT . ";dbname=" . DB_DATABASE . ";charset=utf8mb4";
try {
    $pdo = new PDO($dsn, DB_USERNAME, DB_PASSWORD, $options);
} catch (PDOException $e) {
    http_response_code(500);
    echo json_encode(['error' => "Error de conexión: " . $e->getMessage()]);
    exit;
}

// Autenticación básica
if (!isset($_SERVER['PHP_AUTH_USER'])) {
    header('WWW-Authenticate: Basic realm="Mi API"');
    header('HTTP/1.0 401 Unauthorized');
    echo json_encode(['error' => 'Autenticación requerida.']);
    exit;
} else {
    if ($_SERVER['PHP_AUTH_USER'] !== 'Super' || $_SERVER['PHP_AUTH_PW'] !== 'Lmas2020!1') {
        header('HTTP/1.0 401 Unauthorized');
        echo json_encode(['error' => 'Credenciales no válidas.']);
        exit;
    }
}

if ($_SERVER['REQUEST_METHOD'] === 'DELETE') {
    if (isset($_GET['sectionid'], $_GET['prodid'], $_GET['optsessionid'], $_GET['optid'])) {
        $sectionid = $_GET['sectionid'];
        $prodid = $_GET['prodid'];
        $optsessionid = $_GET['optsessionid'];
        $optid = $_GET['optid'];
        $sql = "DELETE FROM `ocow_optionsaved` WHERE `sectionid` = :sectionid AND `prodid` = :prodid AND `optsessionid` = :optsessionid AND `optid` = :optid";
        $params = ['sectionid' => $sectionid, 'prodid' => $prodid, 'optsessionid' => $optsessionid, 'optid' => $optid];

        $stmt = $pdo->prepare($sql);
        $stmt->execute($params);
        $rowCount = $stmt->rowCount();

        if ($rowCount > 0) {
            echo json_encode(['success' => "Registros eliminados: $rowCount"]);
        } else {
            echo json_encode(['error' => 'No se encontraron datos para eliminar.']);
        }
    } else {
        http_response_code(400);
        echo json_encode(['error' => 'Faltan los parámetros requeridos: sectionid, prodid, optsessionid, y optid.']);
    }
} else {
    http_response_code(405);
    echo json_encode(['error' => 'Método HTTP no soportado']);
}
?>
