<?php
// Configuración de base de datos
require_once('../config.php'); // Ajustar la ruta para incluir config.php desde la raíz

// Habilitar CORS
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: GET, POST, OPTIONS");
header("Access-Control-Allow-Headers: Authorization, Content-Type");

// Manejo de preflight (opcional si necesitas manejar OPTIONS)
if ($_SERVER['REQUEST_METHOD'] === 'OPTIONS') {
    http_response_code(204);
    exit();
}

// Conexión a la base de datos
$conn = new mysqli(DB_HOSTNAME, DB_USERNAME, DB_PASSWORD, DB_DATABASE);

if ($conn->connect_error) {
    die("Connection failed: " . $conn->connect_error);
}

// Obtener la URL base de la tienda
$base_url = defined('HTTPS_SERVER') ? HTTPS_SERVER : HTTP_SERVER;

// Función para verificar el token
function verify_token($token) {
    // Aquí debes implementar la lógica para verificar el token
    // Por ejemplo, consultando la base de datos o utilizando algún sistema de autenticación que ya tengas
    // Devuelve true si el token es válido, de lo contrario, false
    return true;
}

// Verificar si se ha enviado un token en el encabezado de la solicitud
$headers = apache_request_headers();
$token = isset($headers['Authorization']) ? str_replace('Bearer ', '', $headers['Authorization']) : '';

if (!verify_token($token)) {
    http_response_code(401);
    echo json_encode(array('success' => 0, 'error' => array('Unauthorized'), 'data' => array()));
    exit();
}

// Consultar las categorías activas de la base de datos
$sql = "SELECT c.category_id, c.parent_id, cd.name, c.image 
        FROM " . DB_PREFIX . "category c 
        LEFT JOIN " . DB_PREFIX . "category_description cd 
        ON c.category_id = cd.category_id 
        WHERE cd.language_id = 1 AND c.status = 1";  // Añadido c.status = 1 para solo categorías activas

$result = $conn->query($sql);

$categories = array();

if ($result->num_rows > 0) {
    while ($row = $result->fetch_assoc()) {
        $categories[] = array(
            'category_id' => $row['category_id'],
            'parent_id' => $row['parent_id'],
            'name' => $row['name'],
            'seo_url' => '',  // Puedes añadir el SEO URL si está disponible en tu base de datos
            'image' => $base_url . 'image/cache/' . $row['image'],
            'original_image' => $base_url . 'image/' . $row['image'],
            'filters' => array('filter_groups' => array()),  // Puedes añadir filtros si los tienes
            'categories' => array()  // Puedes añadir subcategorías si las tienes
        );
    }
}

$response = array(
    'success' => 1,
    'error' => array(),
    'data' => $categories
);

// Devolver las categorías en formato JSON
header('Content-Type: application/json');
echo json_encode($response);

$conn->close();
?>
