<?php
// Habilitar la visualización de errores
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Habilitar CORS
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type, Authorization");

// Manejar solicitudes OPTIONS (para preflight requests de CORS)
if ($_SERVER['REQUEST_METHOD'] === 'OPTIONS') {
    http_response_code(200);
    exit;
}

// Función para registrar errores en un archivo de log
function log_error($message) {
    file_put_contents('error_log.txt', date('Y-m-d H:i:s') . " - " . $message . "\n", FILE_APPEND);
}

// Incluir el archivo de configuración
require_once('../config.php');

// Crear conexión
$conn = new mysqli(DB_HOSTNAME, DB_USERNAME, DB_PASSWORD, DB_DATABASE, DB_PORT);

// Verificar conexión
if ($conn->connect_error) {
    log_error("La conexión ha fallado: " . $conn->connect_error);
    die(json_encode(["success" => false, "message" => "La conexión ha fallado: " . $conn->connect_error]));
}

// Obtener los datos del cliente desde la solicitud
$data = json_decode(file_get_contents("php://input"), true);

// Verificar los datos recibidos
if (!$data) {
    log_error("Error: Datos de solicitud no válidos o vacíos. Raw input: " . file_get_contents("php://input"));
    die(json_encode(["success" => false, "message" => "Error: Datos de solicitud no válidos o vacíos."]));
}

// Obtener la IP del cliente
$ip = $_SERVER['REMOTE_ADDR'];

// Generar un token y un código
$token = bin2hex(random_bytes(16));
$code = bin2hex(random_bytes(6));

// Campos obligatorios
$customer_group_id = 1; // int
$store_id = 0; // int
$language_id = 1; // int, valor predeterminado
$firstname = $conn->real_escape_string($data['firstname']); // string
$lastname = $conn->real_escape_string($data['lastname']); // string
$email = $conn->real_escape_string($data['email']); // string
$telephone = $conn->real_escape_string($data['telephone']); // string
$fax = ''; // string, valor predeterminado vacío
$salt = substr(md5(uniqid(rand(), true)), 0, 9); // Generar un salt
$password = md5($salt . $data['password']); // string
$custom_field = '[]'; // string, valor predeterminado vacío como JSON
$safe = 1; // int, valor predeterminado
$company = ''; // string, valor predeterminado vacío
$address_1 = $conn->real_escape_string($data['address_1']); // string
$address_2 = ''; // string, valor predeterminado vacío
$city = $conn->real_escape_string($data['city']); // string
$postcode = $conn->real_escape_string($data['postcode']); // string
$country_id = intval($data['country_id']); // int
$zone_id = intval($data['zone_id']); // int

// Registrar datos recibidos
log_error("Datos recibidos: " . json_encode($data));

// Verificar la validez del zone_id para el country_id
$sql_zone_check = "SELECT * FROM ocow_zone WHERE country_id = $country_id AND zone_id = $zone_id";
$result = $conn->query($sql_zone_check);

if ($result->num_rows == 0) {
    log_error("Error: zone_id $zone_id no es válido para country_id $country_id");
    die(json_encode(["success" => false, "message" => "Error: Zone ID no es válido para el Country ID proporcionado."]));
}

// Insertar cliente en la tabla ocow_customer
$sql_customer = "INSERT INTO ocow_customer (
    customer_group_id, store_id, language_id, firstname, lastname, email, telephone, fax, salt, password, custom_field, address_id, status, safe, token, ip, date_added, code
) VALUES (
    $customer_group_id, $store_id, $language_id, '$firstname', '$lastname', '$email', '$telephone', '$fax', '$salt', '$password', '$custom_field', 0, 1, $safe, '$token', '$ip', NOW(), '$code'
)";

log_error("SQL Cliente: " . $sql_customer);

if ($conn->query($sql_customer) === TRUE) {
    $customer_id = $conn->insert_id;

    // Insertar dirección en la tabla ocow_address
    $sql_address = "INSERT INTO ocow_address (customer_id, firstname, lastname, company, address_1, address_2, city, postcode, country_id, zone_id, custom_field)
    VALUES ($customer_id, '$firstname', '$lastname', '$company', '$address_1', '$address_2', '$city', '$postcode', $country_id, $zone_id, '$custom_field')";

    log_error("SQL Dirección: " . $sql_address);

    if ($conn->query($sql_address) === TRUE) {
        $address_id = $conn->insert_id;

        // Actualizar address_id en ocow_customer
        $sql_update_customer = "UPDATE ocow_customer SET address_id = $address_id WHERE customer_id = $customer_id";
        if ($conn->query($sql_update_customer) === TRUE) {
            log_error("Cliente insertado correctamente. ID Cliente: $customer_id, ID Dirección: $address_id");
            echo json_encode(["success" => true, "message" => "Cliente y dirección insertados correctamente.", "customer_id" => $customer_id, "address_id" => $address_id]);
        } else {
            log_error("Error al actualizar el cliente con address_id: " . $conn->error);
            echo json_encode(["success" => false, "message" => "Error al actualizar el cliente con address_id: " . $conn->error]);
        }
    } else {
        log_error("Error al insertar la dirección: " . $conn->error);
        echo json_encode(["success" => false, "message" => "Error al insertar la dirección: " . $conn->error, "sql" => $sql_address]);
    }
} else {
    log_error("Error al insertar el cliente: " . $conn->error);
    echo json_encode(["success" => false, "message" => "Error al insertar el cliente: " . $conn->error, "sql" => $sql_customer]);
}

// Cerrar conexión
$conn->close();
?>
