<?php
// Habilitar la visualización de errores
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Habilitar CORS
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: GET, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type, Authorization");

// Manejar solicitudes OPTIONS (para preflight requests de CORS)
if ($_SERVER['REQUEST_METHOD'] === 'OPTIONS') {
    http_response_code(200);
    exit;
}

// Función para registrar errores en un archivo de log
function log_error($message) {
    file_put_contents(__DIR__ . '/../error_log.txt', date('Y-m-d H:i:s') . " - " . $message . "\n", FILE_APPEND);
}

// Incluir el archivo de configuración
require_once(__DIR__ . '/../config.php');

// Crear conexión
$conn = new mysqli(DB_HOSTNAME, DB_USERNAME, DB_PASSWORD, DB_DATABASE, DB_PORT);

// Verificar conexión
if ($conn->connect_error) {
    log_error("La conexión ha fallado: " . $conn->connect_error);
    die(json_encode(["success" => false, "message" => "La conexión ha fallado: " . $conn->connect_error]));
}

// Obtener fechas de inicio, fin y customer_id de los parámetros de la solicitud
$start_date = isset($_GET['start_date']) ? $conn->real_escape_string($_GET['start_date']) : null;
$end_date = isset($_GET['end_date']) ? $conn->real_escape_string($_GET['end_date']) : null;
$customer_id = isset($_GET['customer_id']) ? intval($_GET['customer_id']) : null;

log_error("Parámetros recibidos - start_date: $start_date, end_date: $end_date, customer_id: $customer_id");

$sql_conditions = [];

// Filtro por fechas si están presentes
if ($start_date && $end_date) {
    $sql_conditions[] = "o.date_added BETWEEN '$start_date' AND '$end_date'";
}

// Filtro por customer_id si está presente
if ($customer_id !== null) {
    $sql_conditions[] = "o.customer_id = $customer_id";
}

// Construcción de la consulta SQL con los filtros
$sql = "
    SELECT o.order_id, o.customer_id, o.date_added, 
           IFNULL(c.firstname, o.firstname) AS firstname, 
           IFNULL(c.lastname, o.lastname) AS lastname, 
           IFNULL(c.email, o.email) AS email,
           o.payment_address_1 AS address_1, o.payment_address_2 AS address_2, o.payment_city AS city, 
           o.payment_postcode AS postcode, o.payment_country_id AS country_id, o.payment_zone_id AS zone_id, 
           z.name as zone_name, z.code as zone_code,
           o.total, o.shipping_method, o.payment_method, o.comment,
           IFNULL(os.name, 'Pending') as order_status
    FROM " . DB_PREFIX . "order o
    LEFT JOIN " . DB_PREFIX . "customer c ON o.customer_id = c.customer_id
    LEFT JOIN " . DB_PREFIX . "zone z ON o.payment_zone_id = z.zone_id
    LEFT JOIN " . DB_PREFIX . "order_status os ON o.order_status_id = os.order_status_id
";

// Añadir condiciones a la consulta
if (count($sql_conditions) > 0) {
    $sql .= " WHERE " . implode(' AND ', $sql_conditions);
}

$sql .= " ORDER BY o.order_id DESC";

// Registrar la consulta SQL para depuración
log_error("Consulta SQL: " . $sql);

$result = $conn->query($sql);

if ($result === false) {
    log_error("Error en la consulta SQL: " . $conn->error);
    die(json_encode(["success" => false, "message" => "Error en la consulta SQL: " . $conn->error]));
}

log_error("Número de filas devueltas: " . $result->num_rows);

if ($result->num_rows > 0) {
    $orders = [];
    while($row = $result->fetch_assoc()) {
        log_error("Fila obtenida: " . json_encode($row)); // Registrar cada fila obtenida para depuración
        
        $orders[] = [
            "order_id" => $row['order_id'],
            "customer_id" => $row['customer_id'],  // Incluyendo el customer_id en la respuesta
            "date_added" => $row['date_added'],
            "firstname" => $row['firstname'] ?? '',
            "lastname" => $row['lastname'] ?? '',
            "email" => $row['email'] ?? '',
            "address_1" => $row['address_1'] ?? '',
            "address_2" => $row['address_2'] ?? '',
            "city" => $row['city'] ?? '',
            "postcode" => $row['postcode'] ?? '',
            "country_id" => $row['country_id'] ?? '',
            "zone_id" => $row['zone_id'] ?? '',
            "zone_name" => $row['zone_name'] ?? '',
            "zone_code" => $row['zone_code'] ?? '',
            "total" => $row['total'],
            "shipping_method" => $row['shipping_method'] ?? '',
            "payment_method" => $row['payment_method'] ?? '',
            "comment" => $row['comment'] ?? '',
            "order_status" => $row['order_status']
        ];
    }

    echo json_encode(["success" => true, "orders" => $orders]);
} else {
    log_error("No se encontraron órdenes.");
    echo json_encode(["success" => false, "message" => "No orders found."]);
}

// Cerrar conexión
$conn->close();
?>
