<?php
// Permitir solicitudes CORS desde cualquier origen
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: GET, POST, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type, Authorization");

// Manejo de preflight requests para CORS
if ($_SERVER['REQUEST_METHOD'] === 'OPTIONS') {
    exit(0);
}

ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

require_once('../config.php');

$required_constants = ['DB_HOSTNAME', 'DB_USERNAME', 'DB_PASSWORD', 'DB_DATABASE', 'DB_PORT', 'DB_PREFIX'];
foreach ($required_constants as $constant) {
    if (!defined($constant)) {
        die("Error: $constant is not defined. Please check your config.php.");
    }
}

$mysqli = new mysqli(DB_HOSTNAME, DB_USERNAME, DB_PASSWORD, DB_DATABASE, DB_PORT);

if ($mysqli->connect_error) {
    die("Connection failed: " . $mysqli->connect_error);
}
error_log("Connected to database manually");

$api_username = 'super';
$api_key = '2020';

if (!isset($_GET['username']) || !isset($_GET['key']) || $_GET['username'] !== $api_username || $_GET['key'] !== $api_key) {
    error_log("Invalid credentials: " . $_GET['username'] . " / " . $_GET['key']);
    echo json_encode(['error' => 'Invalid credentials']);
    exit;
}

error_log("Credentials are valid");

// Filtrar por rango de fechas si se proporcionan
$date_filter = "";
if (isset($_GET['start_date']) && isset($_GET['end_date'])) {
    $start_date = $mysqli->real_escape_string($_GET['start_date']);
    $end_date = $mysqli->real_escape_string($_GET['end_date']);
    $date_filter = " WHERE `date_added` BETWEEN '$start_date' AND '$end_date'";
} else {
    echo json_encode(['error' => 'Both start_date and end_date parameters are required']);
    exit;
}

$order_query = "SELECT * FROM `" . DB_PREFIX . "order`" . $date_filter . " ORDER BY `order_id` DESC";
$order_result = $mysqli->query($order_query);

if ($order_result === false) {
    error_log("Order query failed: " . $mysqli->error);
    echo json_encode(['error' => 'Order query failed: ' . $mysqli->error]);
    exit;
}

error_log("Number of orders found: " . $order_result->num_rows);

if ($order_result->num_rows > 0) {
    $orders = [];
    while ($order = $order_result->fetch_assoc()) {
        $order_id = $order['order_id'];
        $product_query = "SELECT * FROM `" . DB_PREFIX . "order_product` WHERE `order_id` = $order_id";
        $product_result = $mysqli->query($product_query);

        if ($product_result === false) {
            error_log("Product query failed for order ID $order_id: " . $mysqli->error);
            echo json_encode(['error' => "Product query failed for order ID $order_id: " . $mysqli->error]);
            exit;
        }

        $products = [];
        while ($product = $product_result->fetch_assoc()) {
            $products[] = [
                'product_id' => $product['product_id'],
                'name' => $product['name'],
                'model' => $product['model'],
                'quantity' => $product['quantity'],
                'price' => $product['price'],
                'total' => $product['total']
            ];
        }

        $orders[] = [
            'order_id' => $order['order_id'],
            'customer' => $order['firstname'] . ' ' . $order['lastname'],
            'status' => $order['order_status_id'],
            'total' => $order['total'],
            'date_added' => $order['date_added'],
            'products' => $products
        ];
    }
    echo json_encode(['orders' => $orders]);
} else {
    error_log("No orders found");
    echo json_encode(['message' => 'No orders found']);
}

$mysqli->close();
?>
