<?php
// Configuración de CORS
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: GET, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type, Authorization");

// Manejo de solicitudes OPTIONS (preflight)
if ($_SERVER['REQUEST_METHOD'] == 'OPTIONS') {
    header("HTTP/1.1 200 OK");
    exit();
}

// Carga la configuración de OpenCart
require_once('../config.php');

// Verificar encabezado de autenticación
if (!isset($_SERVER['HTTP_AUTHORIZATION'])) {
    header('HTTP/1.1 401 Unauthorized');
    echo json_encode(['success' => 0, 'error' => ['Authorization header missing'], 'data' => null]);
    exit();
}

$auth_header = $_SERVER['HTTP_AUTHORIZATION'];
list($type, $credentials) = explode(' ', $auth_header, 2);

if (strtolower($type) != 'basic') {
    header('HTTP/1.1 401 Unauthorized');
    echo json_encode(['success' => 0, 'error' => ['Invalid authorization type'], 'data' => null]);
    exit();
}

if ($credentials != EXPECTED_CREDENTIALS) {
    header('HTTP/1.1 401 Unauthorized');
    echo json_encode(['success' => 0, 'error' => ['Invalid credentials'], 'data' => null]);
    exit();
}

// Conexión a la base de datos
$connection = new mysqli(DB_HOSTNAME, DB_USERNAME, DB_PASSWORD, DB_DATABASE, DB_PORT);

if ($connection->connect_error) {
    die("Connection failed: " . $connection->connect_error);
}

// Obtener el ID de la orden desde la solicitud
$order_id = isset($_GET['order_id']) ? intval($_GET['order_id']) : 0;

if ($order_id > 0) {
    // Obtener información de la orden
    $order_query = "SELECT * FROM `" . DB_PREFIX . "order` WHERE order_id = " . $order_id;
    $order_result = $connection->query($order_query);

    if ($order_result && $order_result->num_rows > 0) {
        $order_info = $order_result->fetch_assoc();

        // Obtener información del cliente
        $customer_query = "SELECT * FROM `" . DB_PREFIX . "customer` WHERE customer_id = " . $order_info['customer_id'];
        $customer_result = $connection->query($customer_query);
        $customer_info = null;
        if ($customer_result && $customer_result->num_rows > 0) {
            $customer_info = $customer_result->fetch_assoc();

            // Obtener dirección del cliente
            $address_query = "SELECT * FROM `" . DB_PREFIX . "address` WHERE address_id = " . $customer_info['address_id'];
            $address_result = $connection->query($address_query);
            if ($address_result && $address_result->num_rows > 0) {
                $address_info = $address_result->fetch_assoc();
                $customer_info['address'] = [
                    'firstname' => $address_info['firstname'],
                    'lastname' => $address_info['lastname'],
                    'company' => $address_info['company'],
                    'address_1' => $address_info['address_1'],
                    'address_2' => $address_info['address_2'],
                    'city' => $address_info['city'],
                    'postcode' => $address_info['postcode'],
                    'country_id' => $address_info['country_id'],
                    'zone_id' => $address_info['zone_id']
                ];
            }
        }

        // Obtener dirección de envío de la orden
        $shipping_address = [
            'firstname' => $order_info['shipping_firstname'],
            'lastname' => $order_info['shipping_lastname'],
            'company' => $order_info['shipping_company'],
            'address_1' => $order_info['shipping_address_1'],
            'address_2' => $order_info['shipping_address_2'],
            'city' => $order_info['shipping_city'],
            'postcode' => $order_info['shipping_postcode'],
            'country' => $order_info['shipping_country'],
            'zone' => $order_info['shipping_zone']
        ];

        // Obtener el nombre del estado de la orden
        $order_status_query = "SELECT name FROM `" . DB_PREFIX . "order_status` WHERE order_status_id = " . $order_info['order_status_id'] . " AND language_id = " . (int)$order_info['language_id'];
        $order_status_result = $connection->query($order_status_query);
        $order_status_name = '';
        if ($order_status_result && $order_status_result->num_rows > 0) {
            $order_status_info = $order_status_result->fetch_assoc();
            $order_status_name = $order_status_info['name'];
        }

        // Obtener productos de la orden
        $products_query = "SELECT * FROM `" . DB_PREFIX . "order_product` WHERE order_id = " . $order_id;
        $products_result = $connection->query($products_query);

        $products = [];
        $subtotal = 0;
        while ($product = $products_result->fetch_assoc()) {
            // Obtener opciones del producto
            $options_query = "SELECT * FROM `" . DB_PREFIX . "order_option` WHERE order_id = " . $order_id . " AND order_product_id = " . $product['order_product_id'];
            $options_result = $connection->query($options_query);

            $options = [];
            while ($option = $options_result->fetch_assoc()) {
                $options[] = [
                    'name' => $option['name'],
                    'value' => $option['value']
                ];
            }

            // Agregar imagen del producto con el sufijo `-500x500`
            $image_query = "SELECT image FROM `" . DB_PREFIX . "product` WHERE product_id = " . $product['product_id'];
            $image_result = $connection->query($image_query);
            $image = '';
            if ($image_result && $image_result->num_rows > 0) {
                $image_info = $image_result->fetch_assoc();
                if (!empty($image_info['image'])) {
                    $image_parts = pathinfo($image_info['image']);
                    $image = HTTP_SERVER . 'image/cache/' . $image_parts['dirname'] . '/' . $image_parts['filename'] . '-500x500.' . $image_parts['extension'];
                }
            }

            // Calcular el subtotal sumando los totales de cada producto
            $subtotal += $product['total'];

            // Agregar producto formateado a la lista
            $products[] = [
                'key' => $product['order_product_id'],
                'thumb' => $image,
                'name' => $product['name'],
                'points' => 0,
                'product_id' => $product['product_id'],
                'model' => $product['model'],
                'option' => $options,
                'quantity' => $product['quantity'],
                'recurring' => '',
                'stock' => true,
                'reward' => '',
                'price' => '$' . number_format($product['price'], 2),
                'total' => '$' . number_format($product['total'], 2),
                'price_raw' => (float)$product['price'],
                'total_raw' => (float)$product['total']
            ];
        }

        // Calcular el impuesto
        $tax_rate = 0.08625; // Impuesto del 8.625% para el ejemplo
        $tax_total = round($subtotal * $tax_rate, 2); // Calcular el impuesto basado en el subtotal y redondear a 2 decimales

        // Obtener totales de la orden y actualizar con los valores calculados
        $totals = [];
        $grand_total = $subtotal + $tax_total;
        $totals[] = [
            'title' => 'Sub-Total',
            'text' => '$' . number_format($subtotal, 2),
            'value' => $subtotal
        ];
        $totals[] = [
            'title' => 'NY State Tax',
            'text' => '$' . number_format($tax_total, 2),
            'value' => $tax_total
        ];
        $totals[] = [
            'title' => 'Total',
            'text' => '$' . number_format($grand_total, 2),
            'value' => $grand_total
        ];

        // Formatear la respuesta
        $order_data = [
            'order_status_id' => $order_info['order_status_id'],
            'order_status_name' => $order_status_name,
            'weight' => number_format($order_info['weight'], 2) . 'kg',
            'products' => $products,
            'vouchers' => [],
            'coupon_status' => $order_info['coupon_id'] ? '1' : '0',
            'coupon' => $order_info['coupon_id'] ? $order_info['coupon'] : '',
            'voucher_status' => '1',
            'voucher' => '',
            'reward_status' => false,
            'reward' => '',
            'totals' => $totals,
            'tax_total' => '$' . number_format($tax_total, 2),
            'total' => count($products) . ' item(s) - $' . number_format($grand_total, 2),
            'total_raw' => $grand_total,
            'total_product_count' => count($products),
            'has_shipping' => $order_info['shipping_method'] ? 1 : 0,
            'has_download' => $order_info['download'] ? 1 : 0,
            'has_recurring_products' => 0,
            'currency' => [
                'currency_id' => $order_info['currency_id'],
                'symbol_left' => '$',
                'symbol_right' => '',
                'decimal_place' => '2',
                'value' => '1.00000000'
            ],
            'customer' => $customer_info ? [
                'customer_id' => $customer_info['customer_id'],
                'firstname' => $customer_info['firstname'],
                'lastname' => $customer_info['lastname'],
                'email' => $customer_info['email'],
                'telephone' => $customer_info['telephone'],
                'address' => $customer_info['address']
            ] : null,
            'shipping_address' => $shipping_address
        ];

        // Devolver la respuesta en formato JSON
        header('Content-Type: application/json');
        echo json_encode(['success' => 1, 'error' => [], 'data' => $order_data]);
    } else {
        header('Content-Type: application/json');
        echo json_encode(['success' => 0, 'error' => ['Order not found'], 'data' => null]);
    }
} else {
    header('Content-Type: application/json');
    echo json_encode(['success' => 0, 'error' => ['Invalid order ID'], 'data' => null]);
}

$connection->close();
?>
