<?php
// Habilitar la visualizacin de errores
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Habilitar CORS
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type, Authorization");

// Manejar solicitudes OPTIONS (para preflight requests de CORS)
if ($_SERVER['REQUEST_METHOD'] === 'OPTIONS') {
    http_response_code(200);
    exit;
}

// Funcin para registrar errores en un archivo de log
function log_error($message) {
    file_put_contents('error_log.txt', date('Y-m-d H:i:s') . " - " . $message . "\n", FILE_APPEND);
}

// Incluir el archivo de configuracin
require_once('../config.php');

// Verificar la autenticacin de la API
$headers = getallheaders();
$authHeader = isset($headers['Authorization']) ? $headers['Authorization'] : 
              (isset($headers['authorization']) ? $headers['authorization'] : 
              (isset($_SERVER['HTTP_AUTHORIZATION']) ? $_SERVER['HTTP_AUTHORIZATION'] : ''));

// Eliminar espacios en blanco adicionales
$authHeader = trim($authHeader);

// Depuracin adicional: Registrar los valores que se comparan
log_error("Authorization Header Received: " . $authHeader);
log_error("Expected Authorization Header: Basic " . EXPECTED_CREDENTIALS);

// Verificar la autenticacin
if ($authHeader !== 'Basic ' . EXPECTED_CREDENTIALS) {
    log_error("Error de autenticacin: Credenciales no vlidas.");
    echo json_encode(["error" => "No autorizado"]);
    http_response_code(401); // Cdigo de error no autorizado
    exit;
}

// Crear conexin a la base de datos usando las constantes de config.php
$conn = new mysqli(DB_HOSTNAME, DB_USERNAME, DB_PASSWORD, DB_DATABASE, DB_PORT);

// Verificar conexin
if ($conn->connect_error) {
    log_error("La conexin ha fallado: " . $conn->connect_error);
    die(json_encode(["error" => "La conexin ha fallado: " . $conn->connect_error]));
}

// Leer el cuerpo de la solicitud JSON
$inputJSON = file_get_contents('php://input');
$input = json_decode($inputJSON, true);

// Obtener los parmetros de la solicitud JSON
$code = isset($input['code']) ? $input['code'] : null;
$new_password = isset($input['new_password']) ? $input['new_password'] : null;

// Validar que los parmetros no estn vacos
if (empty($code) || empty($new_password)) {
    log_error("Faltan parmetros: code o new_password.");
    echo json_encode(["error" => "Faltan parmetros"]);
    exit;
}

// Consulta SQL para obtener el salt
$sql = "SELECT salt FROM " . DB_PREFIX . "user WHERE code = ?";
$stmt = $conn->prepare($sql);
$stmt->bind_param("s", $code);
$stmt->execute();
$stmt->bind_result($db_salt);
$stmt->fetch();
$stmt->close();

// Verificar si el usuario existe
if (!$db_salt) {
    log_error("Usuario no encontrado para el cdigo: " . $code);
    echo json_encode(["error" => "Usuario no encontrado"]);
    exit;
}

// Generar el hash con el salt obtenido y el nuevo password
$generated_hash = sha1($db_salt . sha1($db_salt . sha1($new_password)));

// Retornar el resultado
echo json_encode(["salt" => $db_salt, "encrypted_password" => $generated_hash]);

// Cerrar conexin
$conn->close();
?>
