<?php
// Configuracin de CORS
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: GET, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type, Authorization");

if ($_SERVER['REQUEST_METHOD'] == 'OPTIONS') {
    header("HTTP/1.1 200 OK");
    exit();
}

ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

require_once('../config.php');

$response = [
    'success' => 0,
    'error' => [],
    'data' => [],
    'total_paid' => 0,
    'debug' => []
];

// Verificar encabezado de autenticacin
if (!isset($_SERVER['HTTP_AUTHORIZATION'])) {
    header('Content-Type: application/json');
    header('HTTP/1.1 401 Unauthorized');
    $response['error'][] = 'Authorization header missing';
    echo json_encode($response);
    exit();
}

$auth_header = $_SERVER['HTTP_AUTHORIZATION'];
list($type, $credentials) = explode(' ', $auth_header, 2);

if (strtolower($type) != 'basic') {
    header('Content-Type: application/json');
    header('HTTP/1.1 401 Unauthorized');
    $response['error'][] = 'Invalid authorization type';
    echo json_encode($response);
    exit();
}

if ($credentials != EXPECTED_CREDENTIALS) {
    header('Content-Type: application/json');
    header('HTTP/1.1 401 Unauthorized');
    $response['error'][] = 'Invalid credentials';
    echo json_encode($response);
    exit();
}

try {
    $connection = new mysqli(DB_HOSTNAME, DB_USERNAME, DB_PASSWORD, DB_DATABASE, DB_PORT);

    if ($connection->connect_error) {
        throw new Exception('Connection failed: ' . $connection->connect_error);
    }
    $response['debug'][] = 'Connected to database';

    $db_prefix = DB_PREFIX;

    // Obtener los parmetros de consulta
    $order_id = isset($_GET['order_id']) ? intval($_GET['order_id']) : 0;
    $customer_id = isset($_GET['customer_id']) ? intval($_GET['customer_id']) : 0;

    // Consulta para obtener los detalles de los pagos
    $query = "SELECT * FROM `" . $db_prefix . "order_payment` WHERE 1";

    if ($order_id > 0) {
        $query .= " AND `order_id` = $order_id";
    }

    if ($customer_id > 0) {
        $query .= " AND `customer_id` = $customer_id";
    }

    $result = $connection->query($query);

    if (!$result) {
        throw new Exception('Query failed: ' . $connection->error);
    }

    $payments = [];
    while ($row = $result->fetch_assoc()) {
        $payments[] = $row;
    }

    // Consulta para obtener el total agregado
    $total_query = "SELECT SUM(amount) as total_amount FROM `" . $db_prefix . "order_payment` WHERE 1";

    if ($order_id > 0) {
        $total_query .= " AND `order_id` = $order_id";
    }

    if ($customer_id > 0) {
        $total_query .= " AND `customer_id` = $customer_id";
    }

    $total_result = $connection->query($total_query);

    if (!$total_result) {
        throw new Exception('Total query failed: ' . $connection->error);
    }

    $total_row = $total_result->fetch_assoc();
    $total_paid = $total_row['total_amount'];

    $response['success'] = 1;
    $response['data'] = $payments;
    $response['total_paid'] = $total_paid;

} catch (Exception $e) {
    $response['error'][] = $e->getMessage();
}

header('Content-Type: application/json');
echo json_encode($response);

if (isset($connection) && $connection instanceof mysqli) {
    $connection->close();
}
?>
