<?php
// Configuración de base de datos
require_once('../config.php'); // Ajustar la ruta para incluir config.php desde la raíz

// Habilitar CORS
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: GET, POST, OPTIONS");
header("Access-Control-Allow-Headers: Authorization, Content-Type");

// Manejo de preflight (opcional si necesitas manejar OPTIONS)
if ($_SERVER['REQUEST_METHOD'] === 'OPTIONS') {
    http_response_code(204);
    exit();
}

// Conexión a la base de datos
$conn = new mysqli(DB_HOSTNAME, DB_USERNAME, DB_PASSWORD, DB_DATABASE);

if ($conn->connect_error) {
    die("Connection failed: " . $conn->connect_error);
}

// Obtener la URL base de la tienda
$base_url = defined('HTTPS_SERVER') ? HTTPS_SERVER : HTTP_SERVER;

// Función para verificar el token
function verify_token($token) {
    // Aquí debes implementar la lógica para verificar el token
    // Por ejemplo, consultando la base de datos o utilizando algún sistema de autenticación que ya tengas
    // Devuelve true si el token es válido, de lo contrario, false
    return true;
}

// Verificar si se ha enviado un token en el encabezado de la solicitud
$headers = apache_request_headers();
$token = isset($headers['Authorization']) ? str_replace('Bearer ', '', $headers['Authorization']) : '';

if (!verify_token($token)) {
    http_response_code(401);
    echo json_encode(array('success' => 0, 'error' => array('Unauthorized'), 'data' => array()));
    exit();
}

// Obtener los parámetros category_id y product_id si están presentes
$category_id = isset($_GET['category_id']) ? intval($_GET['category_id']) : 0;
$product_id = isset($_GET['product_id']) ? intval($_GET['product_id']) : 0;

// Construir la consulta SQL basada en los filtros
if ($product_id > 0) {
    $sql = "SELECT p.product_id, pd.name, m.name as manufacturer, p.sku, p.model, p.image, p.price, p.date_available, p.quantity, p.status, pd.description 
            FROM " . DB_PREFIX . "product p 
            LEFT JOIN " . DB_PREFIX . "product_description pd 
            ON p.product_id = pd.product_id 
            LEFT JOIN " . DB_PREFIX . "manufacturer m 
            ON p.manufacturer_id = m.manufacturer_id 
            WHERE pd.language_id = 1 AND p.product_id = $product_id AND p.status = 1";  // Filtrar por product_id y productos activos
} elseif ($category_id > 0) {
    $sql = "SELECT p.product_id, pd.name, m.name as manufacturer, p.sku, p.model, p.image, p.price, p.date_available, p.quantity, p.status, pd.description 
            FROM " . DB_PREFIX . "product p 
            LEFT JOIN " . DB_PREFIX . "product_description pd 
            ON p.product_id = pd.product_id 
            LEFT JOIN " . DB_PREFIX . "product_to_category pc 
            ON p.product_id = pc.product_id 
            LEFT JOIN " . DB_PREFIX . "manufacturer m 
            ON p.manufacturer_id = m.manufacturer_id 
            WHERE pd.language_id = 1 AND pc.category_id = $category_id AND p.status = 1";  // Filtrar por category_id y productos activos
} else {
    $sql = "SELECT p.product_id, pd.name, m.name as manufacturer, p.sku, p.model, p.image, p.price, p.date_available, p.quantity, p.status, pd.description 
            FROM " . DB_PREFIX . "product p 
            LEFT JOIN " . DB_PREFIX . "product_description pd 
            ON p.product_id = pd.product_id 
            LEFT JOIN " . DB_PREFIX . "manufacturer m 
            ON p.manufacturer_id = m.manufacturer_id 
            WHERE pd.language_id = 1 AND p.status = 1";  // Todos los productos activos
}

$result = $conn->query($sql);

$products = array();

if ($result->num_rows > 0) {
    while ($row = $result->fetch_assoc()) {
        // Obtener las categorías del producto
        $product_id = $row['product_id'];
        $category_sql = "SELECT cd.name, c.category_id FROM " . DB_PREFIX . "category_description cd
                         LEFT JOIN " . DB_PREFIX . "product_to_category pc ON cd.category_id = pc.category_id
                         LEFT JOIN " . DB_PREFIX . "category c ON cd.category_id = c.category_id
                         WHERE pc.product_id = $product_id AND cd.language_id = 1";
        $category_result = $conn->query($category_sql);

        $categories = array();
        if ($category_result->num_rows > 0) {
            while ($category_row = $category_result->fetch_assoc()) {
                $categories[] = array(
                    'name' => $category_row['name'],
                    'id' => $category_row['category_id']
                );
            }
        }

        // Modificar la URL de la imagen para añadir el sufijo -500x500
        $image_parts = pathinfo($row['image']);
        $image_with_suffix = $image_parts['dirname'] . '/' . $image_parts['filename'] . '-500x500.' . $image_parts['extension'];

        $product = array(
            'id' => $row['product_id'],
            'product_id' => $row['product_id'],
            'name' => $row['name'],
            'manufacturer' => $row['manufacturer'],
            'sku' => $row['sku'],
            'model' => $row['model'],
            'image' => $base_url . 'image/cache/' . $image_with_suffix,
            'images' => array(),  // Aquí puedes añadir la lógica para obtener las imágenes adicionales si las tienes
            'original_image' => $base_url . 'image/' . $image_with_suffix,
            'original_images' => array(),  // Aquí puedes añadir la lógica para obtener las imágenes originales adicionales si las tienes
            'price_excluding_tax' => $row['price'],
            'price_excluding_tax_formated' => '$' . number_format($row['price'], 2),
            'price' => $row['price'],
            'price_formated' => '$' . number_format($row['price'], 2),
            'rating' => 0,  // Aquí puedes añadir la lógica para obtener la calificación del producto si la tienes
            'description' => $row['description'],
            'attribute_groups' => array(),  // Aquí puedes añadir la lógica para obtener los grupos de atributos si los tienes
            'special' => 0,  // Aquí puedes añadir la lógica para obtener el precio especial si lo tienes
            'special_excluding_tax' => 0,
            'special_excluding_tax_formated' => '$0.00',
            'special_formated' => '$0.00',
            'special_start_date' => '',
            'special_end_date' => '',
            'discounts' => array(),  // Aquí puedes añadir la lógica para obtener los descuentos si los tienes
            'options' => array(),  // Aquí puedes añadir la lógica para obtener las opciones del producto si las tienes
            'minimum' => '1',
            'meta_title' => $row['name'],
            'meta_description' => '',
            'meta_keyword' => '',
            'seo_url' => '',
            'tag' => $row['name'],
            'upc' => '',
            'ean' => '',
            'jan' => '',
            'isbn' => '',
            'mpn' => '',
            'location' => '',
            'stock_status' => 'In Stock',
            'stock_status_id' => 6,
            'manufacturer_id' => 0,
            'tax_class_id' => 9,
            'date_available' => $row['date_available'],
            'weight' => '0.00000000',
            'weight_class_id' => 1,
            'length' => '0.00000000',
            'width' => '0.00000000',
            'height' => '0.00000000',
            'length_class_id' => 1,
            'subtract' => '1',
            'sort_order' => '1',
            'status' => $row['status'],
            'date_added' => '',
            'date_modified' => '',
            'viewed' => '0',
            'weight_class' => 'kg',
            'length_class' => 'cm',
            'shipping' => '',
            'reward' => null,
            'points' => '0',
            'category' => $categories,
            'quantity' => $row['quantity'],
            'reviews' => array('review_total' => '0'),
            'recurrings' => array()
        );

        $products[] = $product;
    }
}

$response = array(
    'success' => 1,
    'error' => array(),
    'data' => $products
);

// Devolver los productos en formato JSON
header('Content-Type: application/json');
echo json_encode($response);

$conn->close();
?>
