<?php
require_once '../config.php';  // Asegúrate de que la ruta al archivo de configuración es correcta.

header('Content-Type: application/json');
header('Access-Control-Allow-Origin: *');
header('Access-Control-Allow-Methods: GET, POST, OPTIONS');
header('Access-Control-Allow-Headers: Content-Type, Authorization');

if ($_SERVER['REQUEST_METHOD'] === 'OPTIONS') {
    http_response_code(204);
    exit();
}

$options = [
    PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
    PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC,
    PDO::ATTR_EMULATE_PREPARES => false,
];

$dsn = "mysql:host=" . DB_HOSTNAME . ";port=" . DB_PORT . ";dbname=" . DB_DATABASE . ";charset=utf8mb4";
try {
    $pdo = new PDO($dsn, DB_USERNAME, DB_PASSWORD, $options);
} catch (PDOException $e) {
    http_response_code(500);
    echo json_encode(['error' => "Error de conexión: " . $e->getMessage()]);
    exit;
}

if (!isset($_SERVER['PHP_AUTH_USER'])) {
    header('WWW-Authenticate: Basic realm="Mi API"');
    header('HTTP/1.0 401 Unauthorized');
    echo json_encode(['error' => 'Autenticación requerida.']);
    exit;
} else {
    if ($_SERVER['PHP_AUTH_USER'] !== 'Super' || $_SERVER['PHP_AUTH_PW'] !== 'Lmas2020!1') {
        header('HTTP/1.0 401 Unauthorized');
        echo json_encode(['error' => 'Credenciales no válidas.']);
        exit;
    }
}

$method = $_SERVER['REQUEST_METHOD'];

if ($method === 'GET' && isset($_GET['sectionid'], $_GET['optsessionid'])) {
    $sectionid = $_GET['sectionid'];
    $optsessionid = $_GET['optsessionid'];

    if (!is_numeric($optsessionid)) {
        http_response_code(400);
        echo json_encode(['error' => 'optsessionid debe ser un número entero.']);
        exit;
    }

    $sql = "SELECT o.`optionnume`, o.`optid`, o.`optiontype` FROM `ocow_optionsaved` o WHERE o.`sectionid` = :sectionid AND o.`optsessionid` = :optsessionid";
    $stmt = $pdo->prepare($sql);
    $stmt->execute(['sectionid' => $sectionid, 'optsessionid' => $optsessionid]);
    $results = $stmt->fetchAll();

    if (empty($results)) {
        echo json_encode([]);  // Aquí se devuelve un array vacío en lugar de un mensaje de error.
        exit;
    }

    $output = [];

    foreach ($results as $row) {
        if ($row['optiontype'] == 'checkbox') {
            $output[$row['optionnume']][] = $row['optid'];
        } else {
            if (isset($output[$row['optionnume']])) {
                if (!is_array($output[$row['optionnume']])) {
                    $output[$row['optionnume']] = [$output[$row['optionnume']]]; // Convert to array if not already
                }
                $output[$row['optionnume']][] = $row['optid'];
            } else {
                $output[$row['optionnume']] = $row['optid']; // Keep as single value if only one optid
            }
        }
    }

    // Convert single-element arrays back to single values for non-checkbox types
    foreach ($output as $key => $value) {
        if (is_array($value) && count($value) === 1 && $row['optiontype'] != 'checkbox') {
            $output[$key] = $value[0];
        }
    }

    echo json_encode($output);
} else {
    http_response_code(400);
    echo json_encode(['error' => 'Faltan los parámetros sectionid o optsessionid.']);
}
?>
