<?php
// Habilitar la visualización de errores
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Habilitar CORS
header("Access-Control-Allow-Origin: *");
header("Access-Control-Allow-Methods: POST, OPTIONS");
header("Access-Control-Allow-Headers: Content-Type, Authorization");

// Manejar solicitudes OPTIONS (para preflight requests de CORS)
if ($_SERVER['REQUEST_METHOD'] === 'OPTIONS') {
    http_response_code(200);
    exit;
}

// Función para registrar errores en un archivo de log
function log_error($message) {
    file_put_contents('error_log.txt', date('Y-m-d H:i:s') . " - " . $message . "\n", FILE_APPEND);
}

// Incluir el archivo de configuración
require_once('../config.php');

// Verificar la autenticación de la API
$headers = getallheaders();
$authHeader = isset($headers['Authorization']) ? $headers['Authorization'] : 
              (isset($headers['authorization']) ? $headers['authorization'] : 
              (isset($_SERVER['HTTP_AUTHORIZATION']) ? $_SERVER['HTTP_AUTHORIZATION'] : ''));

// Eliminar espacios en blanco adicionales
$authHeader = trim($authHeader);

// Depuración: Registrar los valores que se comparan
log_error("Authorization Header Received: " . $authHeader);
log_error("Expected Authorization Header: Basic " . EXPECTED_CREDENTIALS);

// Verificar la autenticación
if ($authHeader !== 'Basic ' . EXPECTED_CREDENTIALS) {
    log_error("Error de autenticación: Credenciales no válidas.");
    echo json_encode([
        "success" => false,
        "error" => "No autorizado"
    ]);
    http_response_code(401); // Código de error no autorizado
    exit;
}

// Crear conexión a la base de datos usando las constantes de config.php
$conn = new mysqli(DB_HOSTNAME, DB_USERNAME, DB_PASSWORD, DB_DATABASE, DB_PORT);

// Verificar conexión
if ($conn->connect_error) {
    log_error("La conexión ha fallado: " . $conn->connect_error);
    die(json_encode([
        "success" => false,
        "error" => "La conexión ha fallado: " . $conn->connect_error
    ]));
}

// Leer el cuerpo de la solicitud JSON
$inputJSON = file_get_contents('php://input');
$input = json_decode($inputJSON, true);

// Obtener los parámetros de la solicitud JSON
$code = isset($input['code']) ? $input['code'] : null;

// Validar que el parámetro no esté vacío
if (empty($code)) {
    log_error("Falta el parámetro: code.");
    echo json_encode([
        "success" => false,
        "error" => "Falta el parámetro code"
    ]);
    exit;
}

// Consulta SQL para obtener los campos password, salt, username, user_id, user_group_id
$sql = "SELECT password, salt, username, user_id, user_group_id FROM " . DB_PREFIX . "user WHERE code = ?";
$stmt = $conn->prepare($sql);
$stmt->bind_param("s", $code);
$stmt->execute();
$stmt->bind_result($password, $salt, $username, $user_id, $user_group_id);
$stmt->fetch();
$stmt->close();

// Verificar si el usuario existe
if (!$password) {
    log_error("Usuario no encontrado para el código: " . $code);
    echo json_encode([
        "success" => false,
        "error" => "Usuario no encontrado"
    ]);
    exit;
}

// Retornar el resultado
echo json_encode([
    "success" => true,
    "password" => $password,
    "salt" => $salt,
    "username" => $username,
    "user_id" => $user_id,
    "user_group_id" => $user_group_id
]);

// Cerrar conexión
$conn->close();
?>
